@@ -27,6 +27,7 @@ import (
 	"github.com/goplus/gop/parser/parsertest"
 	"github.com/goplus/gop/token"
 	"github.com/goplus/gox"
+	"golang.org/x/tools/go/packages"
 )
 
 var (
@@ -35,7 +36,7 @@ var (
 )
 
 func init() {
-	gox.SetDebug(gox.DbgFlagAll)
+	gox.SetDebug(0)
 	gblFset = token.NewFileSet()
 	baseConf = &cl.Config{
 		Fset:          gblFset,
@@ -300,6 +301,21 @@ func main() {
 `)
 }
 
+func TestEmbeddField(t *testing.T) {
+	gopClTest(t, `import "math/big"
+
+type BigInt struct {
+	*big.Int
+}`, `package main
+
+import big "math/big"
+
+type BigInt struct {
+	Int *big.Int
+}
+`)
+}
+
 func TestAutoProperty(t *testing.T) {
 	gopClTest(t, `import "github.com/goplus/gop/ast/goptest"
 
@@ -355,6 +371,28 @@ func main() {
 `)
 }
 
+// bugfix (only depends order of testing functions)
+// vet: open tutorial/14-Using-goplus-in-Go/foo/gop_autogen.go: no such file or directory
+func TestGopkgDep(t *testing.T) {
+	os.Remove("../tutorial/14-Using-goplus-in-Go/foo/gop_autogen.go")
+	const (
+		loadTypes = packages.NeedImports | packages.NeedDeps | packages.NeedTypes
+		loadModes = loadTypes | packages.NeedName | packages.NeedModule
+	)
+	loadConf := &packages.Config{Mode: loadModes, Fset: gblFset}
+	pkgs, err := baseConf.Ensure().PkgsLoader.Load(
+		loadConf, "github.com/goplus/gop/tutorial/14-Using-goplus-in-Go/gomain")
+	if err != nil {
+		t.Fatal("PkgsLoader.Load failed:", err)
+	}
+	for _, err = range pkgs[0].Errors {
+		t.Fatal("PkgsLoader.Load failed:", err)
+	}
+	if pkgs[0].Name != "main" {
+		t.Fatal("pkg name:", pkgs[0].Name)
+	}
+}
+
 func TestErrWrap(t *testing.T) {
 	gopClTest(t, `
 import "strconv"
@@ -486,6 +524,67 @@ var z builtin.Gop_bigint = x.Gop_Add(y)
 `)
 }
 
+func TestBigIntLit(t *testing.T) {
+	gopClTest(t, `
+var x = 1r
+`, `package main
+
+import (
+	builtin "github.com/goplus/gop/builtin"
+	big "math/big"
+)
+
+var x = builtin.Gop_bigint_Init__1(big.NewInt(1))
+`)
+}
+
+func TestBigRatLit(t *testing.T) {
+	gopClTest(t, `
+var x = 1/2r
+`, `package main
+
+import (
+	builtin "github.com/goplus/gop/builtin"
+	big "math/big"
+)
+
+var x = builtin.Gop_bigrat_Init__2(big.NewRat(1, 2))
+`)
+}
+
+func TestBigRatLitAdd(t *testing.T) {
+	gopClTest(t, `
+var x = 3 + 1/2r
+`, `package main
+
+import (
+	builtin "github.com/goplus/gop/builtin"
+	big "math/big"
+)
+
+var x = builtin.Gop_bigrat_Init__2(big.NewRat(7, 2))
+`)
+}
+
+func TestBigRatAdd(t *testing.T) {
+	gox.SetDebug(gox.DbgFlagAll)
+	gopClTest(t, `
+var x = 3 + 1/2r
+var y = x + 100
+var z = 100 + y
+`, `package main
+
+import (
+	builtin "github.com/goplus/gop/builtin"
+	big "math/big"
+)
+
+var x = builtin.Gop_bigrat_Init__2(big.NewRat(7, 2))
+var y = x.Gop_Add(builtin.Gop_bigrat_Init__0(100))
+var z = builtin.Gop_bigrat_Init__0(100) + y
+`)
+}
+
 func TestTypeConv(t *testing.T) {
 	gopClTest(t, `
 var a = (*struct{})(nil)
@@ -558,6 +657,58 @@ func main() {
 `)
 }
 
+func TestBigIntAssignOp(t *testing.T) {
+	gopClTest(t, `
+var x bigint
+x += 3
+`, `package main
+
+import builtin "github.com/goplus/gop/builtin"
+
+var x builtin.Gop_bigint
+
+func main() {
+	x.Gop_AddAssign(builtin.Gop_bigint_Init__0(3))
+}
+`)
+}
+
+func TestBigIntAssignOp2(t *testing.T) {
+	gopClTest(t, `
+x := 3r
+x *= 2
+`, `package main
+
+import (
+	builtin "github.com/goplus/gop/builtin"
+	big "math/big"
+)
+
+func main() {
+	x := builtin.Gop_bigint_Init__1(big.NewInt(3))
+	x.Gop_MulAssign(builtin.Gop_bigint_Init__0(2))
+}
+`)
+}
+
+func TestBigIntAssignOp3(t *testing.T) {
+	gopClTest(t, `
+x := 3r
+x *= 2r
+`, `package main
+
+import (
+	builtin "github.com/goplus/gop/builtin"
+	big "math/big"
+)
+
+func main() {
+	x := builtin.Gop_bigint_Init__1(big.NewInt(3))
+	x.Gop_MulAssign(builtin.Gop_bigint_Init__1(big.NewInt(2)))
+}
+`)
+}
+
 func TestCompositeLit(t *testing.T) {
 	gopClTest(t, `
 x := []float64{1, 3.4, 5}
@@ -1084,6 +1235,60 @@ func (m M) Foo() {
 `)
 }
 
+func TestOverloadOp(t *testing.T) {
+	gopClTest(t, `
+type foo struct {
+}
+
+func (a *foo) + (b *foo) *foo {
+	println("a + b")
+	return &foo{}
+}
+
+func (a foo) - (b foo) foo {
+	println("a - b")
+	return foo{}
+}
+
+func -(a foo) {
+	println("-a")
+}
+
+func ++(a foo) {
+	println("a++")
+}
+
+var a, b foo
+var c = a - b
+var d = -a
+`, `package main
+
+import fmt "fmt"
+
+type foo struct {
+}
+
+func (a *foo) Gop_Add(b *foo) *foo {
+	fmt.Println("a + b")
+	return &foo{}
+}
+func (a foo) Gop_Sub(b foo) foo {
+	fmt.Println("a - b")
+	return foo{}
+}
+func (a foo) Gop_Neg() {
+	fmt.Println("-a")
+}
+func (a foo) Gop_Inc() {
+	fmt.Println("a++")
+}
+
+var a, b foo
+var c = a.Gop_Sub(b)
+var d = a.Gop_Neg()
+`)
+}
+
 func TestImport(t *testing.T) {
 	gopClTest(t, `import "fmt"
 
