@@ -17,18 +17,21 @@
 package cl_test
 
 import (
+	"os"
 	"testing"
 
 	"github.com/goplus/gop/cl"
 	"github.com/goplus/gop/parser"
 	"github.com/goplus/gop/parser/parsertest"
+	"github.com/goplus/gop/scanner"
 )
 
 func codeErrorTest(t *testing.T, msg, src string) {
 	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", src)
 	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", nil, 0)
 	if err != nil {
-		t.Fatal("ParseFSDir:", err)
+		scanner.PrintError(os.Stderr, err)
+		t.Fatal("parser.ParseFSDir failed")
 	}
 	conf := *baseConf.Ensure()
 	conf.NoFileLine = false
@@ -44,6 +47,19 @@ func codeErrorTest(t *testing.T, msg, src string) {
 	}
 }
 
+func TestErrImport(t *testing.T) {
+	codeErrorTest(t,
+		"./bar.gop:2:13 undefined: testing", `
+func foo(t *testing.T) {
+}`)
+	codeErrorTest(t,
+		"./bar.gop:4:12 testing.Verbose is not a type", `
+import "testing"
+
+func foo(t testing.Verbose) {
+}`)
+}
+
 func TestErrConst(t *testing.T) {
 	codeErrorTest(t,
 		"./bar.gop:3:7 a redeclared in this block\n\tprevious declaration at ./bar.gop:2:5", `
@@ -60,9 +76,9 @@ var a string
 `)
 }
 
-func _TestErrDefineVar(t *testing.T) {
-	codeErrorTest(t,
-		``, `
+func TestErrDefineVar(t *testing.T) {
+	codeErrorTest(t, "./bar.gop:3:1 no new variables on left side of :=\n"+
+		"./bar.gop:3:6 cannot use \"Hi\" (type untyped string) as type int in assignment", `
 a := 1
 a := "Hi"
 `)
