@@ -18,6 +18,7 @@ package cl
 
 import (
 	"log"
+	"math/big"
 	"reflect"
 
 	goast "go/ast"
@@ -290,12 +291,12 @@ func lookupParent(ctx *blockCtx, name string) (types.Object, *gox.PkgRef) {
 
 func compileBasicLit(ctx *blockCtx, v *ast.BasicLit) {
 	if v.Kind == token.RAT {
-		panic("TODO: rational constant")
+		val := v.Value
+		bi, _ := new(big.Int).SetString(val[:len(val)-1], 10) // remove r suffix
+		ctx.cb.UntypedBigInt(bi, v)
+		return
 	}
-	ctx.cb.Val(&goast.BasicLit{
-		Kind:  gotoken.Token(v.Kind),
-		Value: v.Value,
-	}, v)
+	ctx.cb.Val(&goast.BasicLit{Kind: gotoken.Token(v.Kind), Value: v.Value}, v)
 }
 
 const (
@@ -353,7 +354,7 @@ func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit) {
 	var typ, underlying types.Type
 	if v.Type != nil {
 		typ = toType(ctx, v.Type)
-		underlying = getUnderlying(ctx, typ)
+		underlying = typ.Underlying()
 	}
 	kind := checkCompositeLitElts(ctx, v.Elts)
 	if t, ok := underlying.(*types.Struct); ok && kind == compositeLitKeyVal {
