@@ -118,9 +118,8 @@ func toType(ctx *blockCtx, typ ast.Expr) types.Type {
 		return toChanType(ctx, v)
 	case *ast.FuncType:
 		return toFuncType(ctx, v, nil)
-		/*	case *ast.SelectorExpr:
-			return toExternalType(ctx, v)
-		*/
+	case *ast.SelectorExpr:
+		return toExternalType(ctx, v)
 	}
 	log.Panicln("toType: unknown -", reflect.TypeOf(typ))
 	return nil
@@ -134,6 +133,22 @@ var (
 	}
 )
 
+func toChanType(ctx *blockCtx, v *ast.ChanType) *types.Chan {
+	return types.NewChan(typesChanDirs[v.Dir], toType(ctx, v.Value))
+}
+
+func toExternalType(ctx *blockCtx, v *ast.SelectorExpr) types.Type {
+	name := v.X.(*ast.Ident).Name
+	if pkgRef, ok := ctx.imports[name]; ok {
+		o := pkgRef.Ref(v.Sel.Name)
+		if t, ok := o.(*types.TypeName); ok {
+			return t.Type()
+		}
+		panic(ctx.newCodeErrorf(v.Pos(), "%s.%s is not a type", name, v.Sel.Name))
+	}
+	panic(ctx.newCodeErrorf(v.Pos(), "undefined: %s", name))
+}
+
 func toIdentType(ctx *blockCtx, ident string) types.Type {
 	v, _ := lookupParent(ctx, ident)
 	if v == nil {
@@ -148,10 +163,6 @@ func toIdentType(ctx *blockCtx, ident string) types.Type {
 	panic("TODO: not a type")
 }
 
-func toChanType(ctx *blockCtx, v *ast.ChanType) *types.Chan {
-	return types.NewChan(typesChanDirs[v.Dir], toType(ctx, v.Value))
-}
-
 func toStructType(ctx *blockCtx, v *ast.StructType) *types.Struct {
 	pkg := ctx.pkg.Types
 	fieldList := v.Fields.List
@@ -186,6 +197,9 @@ func toFieldTag(v *ast.BasicLit) string {
 }
 
 func getTypeName(typ types.Type) string {
+	if t, ok := typ.(*types.Pointer); ok {
+		typ = t.Elem()
+	}
 	switch t := typ.(type) {
 	case *types.Named:
 		return t.Obj().Name()
