@@ -144,7 +144,7 @@ func compileAssignStmt(ctx *blockCtx, expr *ast.AssignStmt) {
 				log.Panicln("TODO: non-name $v on left side of :=")
 			}
 		}
-		ctx.cb.DefineVarStart(names...)
+		ctx.cb.DefineVarStart(expr.Pos(), names...)
 		defer func() {
 			if e := recover(); e != nil {
 				ctx.cb.ResetInit()
@@ -170,7 +170,7 @@ func compileAssignStmt(ctx *blockCtx, expr *ast.AssignStmt) {
 	if len(expr.Lhs) != 1 || len(expr.Rhs) != 1 {
 		panic("TODO: invalid syntax of assign by operator")
 	}
-	ctx.cb.AssignOp(gotoken.Token(tok))
+	ctx.cb.AssignOp(gotoken.Token(tok), expr)
 }
 
 // forRange(names...) x rangeAssignThen
