@@ -20,10 +20,12 @@ import (
 	"io/ioutil"
 	"os"
 	"path"
+	"reflect"
 	"strings"
 	"testing"
 
 	"github.com/goplus/gop/parser/parsertest"
+	"github.com/goplus/gop/scanner"
 	"github.com/goplus/gop/token"
 	"github.com/qiniu/x/log"
 )
@@ -31,19 +33,24 @@ import (
 // -----------------------------------------------------------------------------
 
 func init() {
-	log.SetOutputLevel(log.Ldebug)
-	log.SetFlags(log.Llevel)
+	log.SetFlags(log.Llongfile)
+	SetDebug(DbgFlagAll)
 }
 
 func testFrom(t *testing.T, pkgDir, sel string) {
 	if sel != "" && !strings.Contains(pkgDir, sel) {
 		return
 	}
-	log.Debug("Parsing", pkgDir)
+	log.Println("Parsing", pkgDir)
 	fset := token.NewFileSet()
-	pkgs, err := ParseDir(fset, pkgDir, nil, 0)
+	pkgs, err := ParseDir(fset, pkgDir, nil, Trace)
 	if err != nil || len(pkgs) != 1 {
-		t.Fatal("ParseDir failed:", err, len(pkgs))
+		if errs, ok := err.(scanner.ErrorList); ok {
+			for _, e := range errs {
+				t.Log(e)
+			}
+		}
+		t.Fatal("ParseDir failed:", err, reflect.TypeOf(err), len(pkgs))
 	}
 	for _, pkg := range pkgs {
 		b, err := ioutil.ReadFile(pkgDir + "/parser.expect")
@@ -67,7 +74,11 @@ func TestFromTestdata(t *testing.T) {
 		t.Fatal("ReadDir failed:", err)
 	}
 	for _, fi := range fis {
-		testFrom(t, dir+"/"+fi.Name(), sel)
+		name := fi.Name()
+		if strings.HasPrefix(name, "_") {
+			continue
+		}
+		testFrom(t, dir+"/"+name, sel)
 	}
 }
 
